﻿using System;
using System.Windows.Forms;
using System.Collections.ObjectModel;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;
using System.Net;
using System.Data;
using System.IO;

namespace  GPStoGoogleMapDesktop
{
    public partial class TestForm : Form
    {
        private System.Windows.Forms.Timer timer = new Timer();
        private StreamReader mGPSData = new StreamReader(@"..\..\GPSinfo.txt");
        InMemoryFeatureLayer pointLayer = new InMemoryFeatureLayer();
        LayerOverlay pointOverlay = new LayerOverlay();

        public TestForm()
        {
            InitializeComponent();
        }

        private void TestForm_Load(object sender, EventArgs e)
        {
            timer.Interval = 1000;
            timer.Tick += new EventHandler(timer_Tick);
            winformsMap1.MapUnit = GeographyUnit.Meter;
            winformsMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.FromArgb(255, 198, 255, 255));
            WebProxy WP = new WebProxy("proxy.crc.ca");
            WebRequest.DefaultWebProxy = WP;
            GoogleMapsOverlay googleOverlay = new GoogleMapsOverlay(); //(@"Insert your key here!", @"C:\GoogleCache");
            googleOverlay.WebProxy = WP;
            googleOverlay.MapType = GoogleMapsMapType.RoadMap;
            //winformsMap1.Overlays.Add(googleOverlay);
            googleOverlay.DrawingExceptionMode = DrawingExceptionMode.DrawException;
            // This sets the zoom levels to map to Googles.  We next make sure we snap to the zoomlevels
            winformsMap1.ZoomLevelSet = new GoogleMapZoomLevelSet();

            OpenStreetMapOverlay OpenStreet = new OpenStreetMapOverlay();
            OpenStreet.WebProxy = WP;
            winformsMap1.Overlays.Add(OpenStreet);
                

            pointLayer.Open();
            pointLayer.Columns.Add(new FeatureSourceColumn("Text"));
            pointLayer.Close();

            //Sets the projection parameters to go from Geodetic (EPSG 4326) or decimal degrees to Google Map projection (Spherical Mercator).
            Proj4Projection proj4 = new Proj4Projection();
            proj4.InternalProjectionParametersString = Proj4Projection.GetEpsgParametersString(4326);
            proj4.ExternalProjectionParametersString = Proj4Projection.GetGoogleMapParametersString();
            //Applies the projection to the InMemoryFeatureLayer so that the point in decimal degrees (Longitude/Latitude) can be 
            //match the projection of Google Map.
            pointLayer.FeatureSource.Projection = proj4;

            //Values in Longitude and Latitude.
            double Longitude = -95.2809;
            double Latitude = 38.9543;


            //Creates the feature made of a PointShape with the Longitude and Latitude values.
            Feature GPSFeature = new Feature(new PointShape(Longitude, Latitude));

            //Format the Longitude and Latitude into a nice string as Degrees Minutes and Seconds
            string LongLat = DecimalDegreesHelper.GetDegreesMinutesSecondsStringFromDecimalDegreePoint(GPSFeature);

            //Sets the InMemoryFeatureLayer to have it displayed with Square symbol and with text.
            GPSFeature.ColumnValues.Add("Text", LongLat);
            pointLayer.InternalFeatures.Add(GPSFeature);

            pointLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.CreateSimplePointStyle(PointSymbolType.Square,
                                                                    GeoColor.StandardColors.Red, GeoColor.StandardColors.Black, 2, 12);
            pointLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = TextStyles.CreateSimpleTextStyle("Text", "Arial", 12, DrawingFontStyles.Bold,
                                                                    GeoColor.StandardColors.Black, GeoColor.StandardColors.White, 3, -10, 10);
            pointLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;


            pointOverlay.Layers.Add("PointLayer", pointLayer);
            winformsMap1.Overlays.Add("PointOverlay", pointOverlay);

            
            //Sets the extend of the map based on the GPS point.
            proj4.Open();
            Vertex projVertex = proj4.ConvertToExternalProjection(Longitude, Latitude);
            proj4.Close();

            double extendWidth = 2300;
            double extendHeight = 1200;

            winformsMap1.CurrentExtent = new RectangleShape((projVertex.X - extendWidth), (projVertex.Y + extendHeight),
                (projVertex.X + extendWidth), (projVertex.Y - extendHeight));
            
            winformsMap1.Refresh();


        }

        void timer_Tick(object sender, EventArgs e)
        {
            //Gets the GPS info from the textfile.
            DataTable carData = GetCarData();
            
            // Get the Row of Data we are working with.
            DataRow carDataRow = carData.Rows[0];

            double Lat = Convert.ToDouble(carDataRow["LAT"]);
            double Long = Convert.ToDouble(carDataRow["LONG"]);
            
           // MessageBox.Show(string.Format("{0},{1}", Lat,Long));


            Proj4Projection proj4 = new Proj4Projection();
            proj4.InternalProjectionParametersString = Proj4Projection.GetEpsgParametersString(4326);
            proj4.ExternalProjectionParametersString = Proj4Projection.GetGoogleMapParametersString();
            //Applies the projection to the InMemoryFeatureLayer so that the point in decimal degrees (Longitude/Latitude) can be 
            //match the projection of Google Map.
            pointLayer.FeatureSource.Projection = proj4;

            Feature GPSFeature = new Feature(new PointShape(Long, Lat));

            //Format the Longitude and Latitude into a nice string as Degrees Minutes and Seconds
            string LongLat = DecimalDegreesHelper.GetDegreesMinutesSecondsStringFromDecimalDegreePoint(GPSFeature);

            //Sets the InMemoryFeatureLayer to have it displayed with Square symbol and with text.
            
            GPSFeature.ColumnValues.Add("Text", LongLat);


            pointLayer.FeatureSource.Open();
            pointLayer.EditTools.BeginTransaction();
            pointLayer.FeatureSource.DeleteFeature("GPSFeature");
            pointLayer.EditTools.CommitTransaction();
            pointLayer.FeatureSource.Close();

         
            //pointLayer.InternalFeatures.Clear();
            pointLayer.InternalFeatures.Remove(GPSFeature);
            pointLayer.InternalFeatures.Add(GPSFeature);


            pointOverlay.Layers.Add(pointLayer);
            winformsMap1.Overlays.Remove(pointOverlay);
            winformsMap1.Overlays.Add(pointOverlay);

           
            //Function to center the map to a point (the moving moving vehicle feature)
            //winformsMap1.CenterAt((float)Long, (float)Lat);

            winformsMap1.Refresh();

        }

        private DataTable GetCarData()
        {
            DataTable datatable = new DataTable();
            datatable.Columns.Add("LAT");
            datatable.Columns.Add("LONG");

            string strLattitude = "";
            string strLongitude = "";

            // Read the next line from the text file with GPS data in it.
            string strCurrentText = mGPSData.ReadLine();

            if (strCurrentText == "")
            {
                mGPSData.BaseStream.Seek(0, SeekOrigin.Begin);
                strCurrentText = mGPSData.ReadLine();
            }

            while (strCurrentText != null)
            {
                // Every other line is a "/" and we want to skip those.
                if (strCurrentText.Trim() != "/")
                {
                    string[] strSplit = strCurrentText.Split(','); // (':');
                    strLongitude = strSplit[0];
                    strLattitude = strSplit[1];
                    break;
                }
                strCurrentText = mGPSData.ReadLine();
            }

            object[] objs = new object[2] { strLattitude, strLongitude };

            datatable.Rows.Add(objs);

            return datatable;
        }
        private void winformsMap1_MouseMove(object sender, MouseEventArgs e)
        {
            //Displays the X and Y in screen coordinates.
            statusStrip1.Items["toolStripStatusLabelScreen"].Text = "X:" + e.X + " Y:" + e.Y;

            //Gets the PointShape in world coordinates from screen coordinates.
            PointShape pointShape = ExtentHelper.ToWorldCoordinate(winformsMap1.CurrentExtent, new ScreenPointF(e.X, e.Y), winformsMap1.Width, winformsMap1.Height);

            //Displays world coordinates.
            statusStrip1.Items["toolStripStatusLabelWorld"].Text = "(world) X:" + Math.Round(pointShape.X, 4) + " Y:" + Math.Round(pointShape.Y, 4);
        }
        
        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox1.Checked == true)
            {
                timer.Start();
            }

            if (checkBox1.Checked == false)
            {
                timer.Stop();
            }
        }

    }
}
